"use strict";

/* exported logger */
var Promise = require("bluebird");

var AbstractGuiElement = require('../AbstractGuiElement');
var GuiConnector = require('../../GuiConnector');
var ValidationError = require('../../ValidationError');

var Functions = require('../../Functions');
// noinspection JSUnusedLocalSymbols
var logger = require('../../logger');

var guiUtils = new (require('../leftPanel/GuiUtils'))();

function EditUserDialog(params) {
  AbstractGuiElement.call(this, params);
  var self = this;
  self.setUser(params.user);

  self.createGui();
  self.registerListenerType("onSave");
}

EditUserDialog.prototype = Object.create(AbstractGuiElement.prototype);
EditUserDialog.prototype.constructor = EditUserDialog;

EditUserDialog.prototype.setUser = function (user) {
  this._user = user;
  this.setIsNewUser(user.getLogin() === undefined);
};
EditUserDialog.prototype.getUser = function () {
  return this._user;
};

EditUserDialog.prototype.setIsNewUser = function (isNewUser) {
  this._isNewUser = isNewUser;
};
EditUserDialog.prototype.getIsNewUser = function () {
  return this._isNewUser;
};

EditUserDialog.prototype.createGui = function () {
  var self = this;
  var element = self.getElement();

  var tabDiv = Functions.createElement({
    type: "div",
    name: "tabView",
    className: "tabbable boxed parentTabs"
  });
  element.appendChild(tabDiv);

  var tabMenuDiv = Functions.createElement({
    type: "ul",
    className: "nav nav-tabs"
  });
  tabDiv.appendChild(tabMenuDiv);

  var tabContentDiv = Functions.createElement({
    type: "div",
    className: "tab-content"
  });
  tabDiv.appendChild(tabContentDiv);

  self.createGeneralTab(tabMenuDiv, tabContentDiv);
  self.createPrivilegesTab(tabMenuDiv, tabContentDiv);
  self.createProjectsTab(tabMenuDiv, tabContentDiv);
};

EditUserDialog.prototype.createGeneralTab = function (tabMenuDiv, tabContentDiv) {
  var self = this;
  self.addTab({
    tabMenuDiv: tabMenuDiv,
    tabContentDiv: tabContentDiv,
    name: "GENERAL",
    content: self.createGeneralTabContent()
  });

};

var login_counter = 0;

EditUserDialog.prototype.generateTabId = function (tab_name) {
  var self = this;
  var login = self.getUser().getLogin();
  if (login === undefined) {
    login = "new_user_" + (login_counter++);
  }
  return login.replace(".", "_") + tab_name + "_TAB";
};

EditUserDialog.prototype.createPrivilegesTab = function (tabMenuDiv, tabContentDiv) {
  var self = this;
  self.addTab({
    tabMenuDiv: tabMenuDiv,
    tabContentDiv: tabContentDiv,
    name: "PRIVILEGES",
    content: self.createPrivilegesTabContent()
  });

};

EditUserDialog.prototype.addTab = function (params) {
  var id = this.generateTabId(params.name);

  var navLi = guiUtils.createTabMenuObject({
    id: id,
    name: params.name,
    navigationBar: params.tabMenuDiv
  });
  params.tabMenuDiv.appendChild(navLi);

  var contentDiv = guiUtils.createTabContentObject({
    id: id,
    navigationObject: navLi,
    navigationBar: params.tabMenuDiv
  });

  if (params.content !== undefined) {
    contentDiv.appendChild(params.content);
  }

  params.tabContentDiv.appendChild(contentDiv);
};

function getStringIfDefined(value) {
  if (value === undefined) {
    return "";
  }
  return value;
}

EditUserDialog.prototype.createGeneralTabContent = function () {
  var self = this;
  var user = self.getUser();

  var result = new Functions.createElement({
    type: "div"
  });

  var table = new Functions.createElement({
    type: "div",
    style: "display:table"
  });
  result.appendChild(table);

  var loginRow = new Functions.createElement({
    type: "div",
    style: "display:table-row"
  });
  table.appendChild(loginRow);
  loginRow.appendChild(new Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "Login"
  }));
  if (user.getLogin() === undefined) {
    loginRow.appendChild(new Functions.createElement({
      type: "div",
      style: "display:table-cell",
      content: "<input name='userLogin' value=''/>"
    }));
  } else {
    loginRow.appendChild(new Functions.createElement({
      type: "div",
      style: "display:table-cell",
      content: "<input name='userLogin' value='" + getStringIfDefined(user.getLogin()) + "' readonly/>"
    }));
  }

  var passwordRow = new Functions.createElement({
    type: "div",
    style: "display:table-row"
  });
  table.appendChild(passwordRow);
  passwordRow.appendChild(new Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "Password"
  }));
  passwordRow.appendChild(new Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "<input type=\"password\" name='userPassword' value=''/>"
  }));

  var passwordRow2 = new Functions.createElement({
    type: "div",
    style: "display:table-row"
  });
  table.appendChild(passwordRow2);
  passwordRow2.appendChild(new Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "Confirm password"
  }));
  passwordRow2.appendChild(new Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "<input type=\"password\" name='userPassword2' value=''/>"
  }));

  var nameRow = new Functions.createElement({
    type: "div",
    style: "display:table-row"
  });
  table.appendChild(nameRow);
  nameRow.appendChild(new Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "Name"
  }));
  nameRow.appendChild(new Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "<input name='userName' value='" + getStringIfDefined(user.getName()) + "'/>"
  }));

  var surnameRow = new Functions.createElement({
    type: "div",
    style: "display:table-row"
  });
  table.appendChild(surnameRow);
  surnameRow.appendChild(new Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "Surname"
  }));
  surnameRow.appendChild(new Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "<input name='userSurname' value='" + getStringIfDefined(user.getSurname()) + "'/>"
  }));

  var emailRow = new Functions.createElement({
    type: "div",
    style: "display:table-row"
  });
  table.appendChild(emailRow);
  emailRow.appendChild(new Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "Email"
  }));
  emailRow.appendChild(new Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "<input name='userEmail' value='" + getStringIfDefined(user.getEmail()) + "'/>"
  }));


  var menuRow = Functions.createElement({
    type: "div",
    className: "minerva-menu-row",
    style: "display:table-row; margin:10px"
  });
  result.appendChild(menuRow);

  var saveUserButton = Functions.createElement({
    type: "button",
    name: "saveUser",
    content: '<span class="ui-icon ui-icon-disk"></span>&nbsp;SAVE',
    onclick: function () {
      return self.onSaveClicked().then(function () {
        return self.close();
      }, GuiConnector.alert);
    }
  });
  var cancelButton = Functions.createElement({
    type: "button",
    name: "cancelProject",
    content: '<span class="ui-icon ui-icon-cancel"></span>&nbsp;CANCEL',
    onclick: function () {
      return self.close();
    }
  });
  menuRow.appendChild(saveUserButton);
  menuRow.appendChild(cancelButton);

  return result;

};

EditUserDialog.prototype.createPrivilegesTabContent = function () {
  var self = this;
  var result = Functions.createElement({
    type: "div"
  });
  result.appendChild(self._createPrivilegesTable());
  return result;
};

EditUserDialog.prototype._createPrivilegesTable = function () {
  var result = Functions.createElement({
    type: "div",
    style: "margin-top:10px;"
  });

  var privilegesTable = Functions.createElement({
    type: "table",
    name: "privilegesTable",
    className: "display",
    style: "width:100%"
  });
  result.appendChild(privilegesTable);

  return result;
};


EditUserDialog.prototype.createProjectsTab = function (tabMenuDiv, tabContentDiv) {
  var self = this;
  self.addTab({
    tabMenuDiv: tabMenuDiv,
    tabContentDiv: tabContentDiv,
    name: "PROJECTS",
    content: self.createProjectsTabContent()
  });
};

EditUserDialog.prototype.createProjectsTabContent = function () {
  var self = this;
  var result = Functions.createElement({
    type: "div"
  });
  result.appendChild(self._createProjectsTable());
  return result;
};

EditUserDialog.prototype._createProjectsTable = function () {
  var result = Functions.createElement({
    type: "div",
    style: "margin-top:10px;"
  });

  var projectsTable = Functions.createElement({
    type: "table",
    name: "projectsTable",
    className: "display",
    style: "width:100%"
  });
  result.appendChild(projectsTable);

  return result;
};

EditUserDialog.prototype.init = function () {
  var self = this;
  return self.initProjectsTab().then(function () {
    return self.refreshProjects();
  }).then(function () {
    return self.initPrivilegesTab();
  }).then(function () {
    $(window).trigger('resize');
  });
};

EditUserDialog.prototype.initProjectsTab = function () {
  var self = this;

  var usersTable = $("[name=projectsTable]", self.getElement())[0];

  var configuration;
  return ServerConnector.getConfiguration().then(function (result) {
    configuration = result;
    return self.createUserPrivilegeColumns()
  }).then(function (columns) {
    $(usersTable).DataTable({
      columns: columns
    });
    $(usersTable).on("click", "[name='project-privilege-checkbox']", function () {
      var data = $(this).attr("data").split("-");
      var privilegeType = data[0];
      var objectId = data[1];
      var value = $(this).is(":checked");
      for (var i = 0; i < configuration.getPrivilegeTypes().length; i++) {
        var privilege = configuration.getPrivilegeTypes()[i];
        if (privilege.getName() === privilegeType) {
          self.getUser().setPrivilege({type: privilege, value: value, objectId: objectId});
        }
      }
    });

  });
};

EditUserDialog.prototype.initPrivilegesTab = function () {
  var self = this;

  var usersTable = $("[name=privilegesTable]", self.getElement())[0];

  return ServerConnector.getConfiguration().then(function (configuration) {
    var columns = [{
      title: "Name"
    }, {
      title: "Value"
    }];
    var dataTable = $(usersTable).DataTable({
      columns: columns
    });
    var data = [];

    for (var i = 0; i < configuration.getPrivilegeTypes().length; i++) {
      var privilege = configuration.getPrivilegeTypes()[i];
      if (privilege.getObjectType() === null) {
        if (privilege.getValueType() === "boolean") {
          var checked = "";
          if (self.getUser().hasPrivilege(privilege)) {
            checked = "checked"
          }
          data.push([
            "<span>" + privilege.getCommonName() + "</span>",
            "<input type='checkbox' name='privilege-checkbox' data='" + privilege.getName() + "' " + checked + " />"
          ]);
        } else if (privilege.getValueType() === "int") {
          data.push([
            "<span>" + privilege.getCommonName() + "</span>",
            "<input name='privilege-int' data='" + privilege.getName() + "' value='" + self.getUser().getPrivilegeValue(privilege) + "' />"
          ]);

        }
      }
    }
    dataTable.clear().rows.add(data).draw();
    $(usersTable).on("click", "[name='privilege-checkbox']", function () {
      var privilegeType = $(this).attr("data");
      for (var i = 0; i < configuration.getPrivilegeTypes().length; i++) {
        var privilege = configuration.getPrivilegeTypes()[i];
        if (privilege.getName() === privilegeType) {
          self.getUser().setPrivilege({type: privilege, value: $(this).is(":checked")});
        }
      }
    });
    $(usersTable).on("change", "[name='privilege-int']", function () {
      var privilegeType = $(this).attr("data");
      var value = $(this).val();
      value = parseInt(value);
      if (Functions.isInt(value)) {
        for (var i = 0; i < configuration.getPrivilegeTypes().length; i++) {
          var privilege = configuration.getPrivilegeTypes()[i];
          if (privilege.getName() === privilegeType) {
            self.getUser().setPrivilege({type: privilege, value: value});
          }
        }
      }
    });
  });
};

EditUserDialog.prototype.refreshProjects = function () {
  var self = this;
  return ServerConnector.getProjects().then(function (projects) {
    return self.setProjects(projects);
  });
};

EditUserDialog.prototype.setProjects = function (projects) {
  var self = this;
  self._userByLogin = [];
  return self.createUserPrivilegeColumns().then(function (columns) {
    var dataTable = $($("[name='projectsTable']", self.getElement())[0]).DataTable();
    var data = [];
    for (var i = 0; i < projects.length; i++) {
      var project = projects[i];
      var rowData = self.projectToTableRow(project, columns);
      data.push(rowData);
    }
    dataTable.clear().rows.add(data).draw();
  });
};

EditUserDialog.prototype.projectToTableRow = function (project, columns) {
  var user = this.getUser();
  var row = [];

  row[0] = "<span data='" + project.getId() + "'>" + project.getProjectId() + "</span>";

  for (var i = 1; i < columns.length; i++) {
    var privilege = columns[i].privilegeType;
    var checked = "";
    if (user.hasPrivilege(privilege, project.getId())) {
      checked = "checked";
    }
    row.push("<input type='checkbox' name='project-privilege-checkbox' data='" + privilege.getName() + "-" + project.getId() + "' " + checked + " />");
  }

  return row;
};


EditUserDialog.prototype.destroy = function () {
  var self = this;
  var div = self.getElement();
  var usersTable = $("[name=projectsTable]", self.getElement())[0];
  if ($.fn.DataTable.isDataTable(usersTable)) {
    $(usersTable).DataTable().destroy();
  }
  var privilegesTable = $("[name=privilegesTable]", self.getElement())[0];
  if ($.fn.DataTable.isDataTable(privilegesTable)) {
    $(privilegesTable).DataTable().destroy();
  }


  if ($(div).hasClass("ui-dialog-content")) {
    $(div).dialog("destroy");
  }
};

EditUserDialog.prototype.open = function () {
  var self = this;
  var div = self.getElement();
  var title = self.getUser().getLogin();
  if (title === undefined) {
    title = "NEW USER";
  }
  if (!$(div).hasClass("ui-dialog-content")) {
    $(div).dialog({
      title: title,
      width: window.innerWidth / 2,
      height: window.innerHeight / 2
    });
  }
  $(div).dialog("open");
};

EditUserDialog.prototype.onSaveClicked = function () {
  var self = this;
  var user = self.getUser();
  return self.checkValidity().then(function () {
    user.setLogin(self.getLogin());
    user.setPassword(self.getPassword());
    user.setEmail(self.getEmail());
    user.setName(self.getName());
    user.setSurname(self.getSurname());
    if (self.getIsNewUser()) {
      return ServerConnector.addUser(user);
    } else {
      return ServerConnector.updateUser(user);
    }
  }).then(function () {
    return self.callListeners("onSave", user);
  });
};

EditUserDialog.prototype.checkValidity = function () {
  var self = this;
  var isValid = true;
  var error = "<b>Some data is missing.</b><ul>";
  if (self.getPassword() !== self.getPassword2()) {
    error += "<li>Password doesn't match</li>";
    isValid = false;
  }
  if (self.getLogin() === "" || self.getLogin() === undefined || self.getLogin() === null) {
    error += "<li>Login must not be empty</li>";
    isValid = false;
  }
  var promise = Promise.resolve();
  if (self.getUser().getLogin() !== self.getLogin()) {
    promise = ServerConnector.getUser(self.getLogin()).then(function (remoteUser) {
      if (remoteUser !== null) {
        error += "<li>Login already used</li>";
        isValid = false;
      }
    });
  }
  return promise.then(function () {
    if (isValid) {
      return Promise.resolve(true);
    } else {
      return Promise.reject(new ValidationError(error));
    }
  });
};

EditUserDialog.prototype.getPassword = function () {
  var self = this;
  return $("[name='userPassword']", self.getElement()).val();
};
EditUserDialog.prototype.getPassword2 = function () {
  var self = this;
  return $("[name='userPassword2']", self.getElement()).val();
};

EditUserDialog.prototype.getLogin = function () {
  var self = this;
  return $("[name='userLogin']", self.getElement()).val();
};

EditUserDialog.prototype.getEmail = function () {
  var self = this;
  return $("[name='userEmail']", self.getElement()).val();
};
EditUserDialog.prototype.getName = function () {
  var self = this;
  return $("[name='userName']", self.getElement()).val();
};
EditUserDialog.prototype.getSurname = function () {
  var self = this;
  return $("[name='userSurname']", self.getElement()).val();
};


EditUserDialog.prototype.close = function () {
  var self = this;
  $(self.getElement()).dialog("close");
};

EditUserDialog.prototype.createUserPrivilegeColumns = function () {
  var self = this;

  if (self._userPrivilegeColumns !== undefined) {
    return Promise.resolve(self._userPrivilegeColumns);
  }

  return ServerConnector.getConfiguration().then(function (configuration) {
    self._userPrivilegeColumns = [{
      title: "ProjectId"
    }];
    var privilegeTypes = configuration.getPrivilegeTypes();
    for (var i = 0; i < privilegeTypes.length; i++) {
      var type = privilegeTypes[i];
      if (type.getObjectType() === "Project") {
        self._userPrivilegeColumns.push({
          "title": type.getCommonName(),
          privilegeType: type
        });
      }
    }
    return self._userPrivilegeColumns;
  });

};


module.exports = EditUserDialog;
